@@ -5,6 +5,8 @@ import os
 import re
 import logging
 
+from collections import defaultdict
+
 from powerline.lib.threaded import ThreadedSegment
 from powerline.lib.unicode import unicode
 from powerline.lint.markedjson.markedvalue import MarkedUnicode
@@ -31,7 +33,7 @@ generic_keys = set((
 ))
 type_keys = {
 	'function': set(('function', 'args', 'draw_inner_divider')),
-	'string': set(('contents', 'type', 'highlight_group', 'divider_highlight_group')),
+	'string': set(('contents', 'type', 'highlight_groups', 'divider_highlight_group')),
 	'segment_list': set(('function', 'segments', 'args', 'type')),
 }
 required_keys = {
@@ -39,7 +41,7 @@ required_keys = {
 	'string': set(()),
 	'segment_list': set(('function', 'segments',)),
 }
-highlight_keys = set(('highlight_group', 'name'))
+highlight_keys = set(('highlight_groups', 'name'))
 
 
 def get_function_strings(function_name, context, ext):
@@ -171,41 +173,47 @@ def check_group(group, data, context, echoerr):
 		return True, False, False
 	colorscheme = data['colorscheme']
 	ext = data['ext']
-	configs = []
+	configs = None
 	if ext:
+		def listed_key(d, k):
+			try:
+				return [d[k]]
+			except KeyError:
+				return []
+
 		if colorscheme == '__main__':
-			configs.append([config for config in data['ext_colorscheme_configs'][ext].items()])
-			configs.append([config for config in data['top_colorscheme_configs'].items()])
+			colorscheme_names = set(data['ext_colorscheme_configs'][ext])
+			colorscheme_names.update(data['top_colorscheme_configs'])
+			colorscheme_names.discard('__main__')
+			configs = [
+				(
+					name,
+					listed_key(data['ext_colorscheme_configs'][ext], name)
+					+ listed_key(data['ext_colorscheme_configs'][ext], '__main__')
+					+ listed_key(data['top_colorscheme_configs'], name)
+				)
+				for name in colorscheme_names
+			]
 		else:
-			try:
-				configs.append([data['ext_colorscheme_configs'][ext][colorscheme]])
-			except KeyError:
-				pass
-			try:
-				configs.append([data['ext_colorscheme_configs'][ext]['__main__']])
-			except KeyError:
-				pass
-			try:
-				configs.append([data['top_colorscheme_configs'][colorscheme]])
-			except KeyError:
-				pass
+			configs = [
+				(
+					colorscheme,
+					listed_key(data['ext_colorscheme_configs'][ext], colorscheme)
+					+ listed_key(data['ext_colorscheme_configs'][ext], '__main__')
+					+ listed_key(data['top_colorscheme_configs'], colorscheme)
+				)
+			]
 	else:
 		try:
-			configs.append([data['top_colorscheme_configs'][colorscheme]])
+			configs = [(colorscheme, [data['top_colorscheme_configs'][colorscheme]])]
 		except KeyError:
 			pass
-	new_echoerr = DelayedEchoErr(echoerr)
 	hadproblem = False
-	for config_lst in configs:
-		tofind = len(config_lst)
+	for new_colorscheme, config_lst in configs:
 		not_found = []
+		new_data = data.copy()
+		new_data['colorscheme'] = new_colorscheme
 		for config in config_lst:
-			if isinstance(config, tuple):
-				new_colorscheme, config = config
-				new_data = data.copy()
-				new_data['colorscheme'] = new_colorscheme
-			else:
-				new_data = data
 			havemarks(config)
 			try:
 				group_data = config['groups'][group]
@@ -220,21 +228,17 @@ def check_group(group, data, context, echoerr):
 				)
 				if chadproblem:
 					hadproblem = True
-				else:
-					tofind -= 1
-					if not tofind:
-						return proceed, echo, hadproblem
 				if not proceed:
 					break
-		if not_found:
-			new_echoerr(
+		if not_found and len(not_found) == len(config_lst):
+			echoerr(
 				context='Error while checking group definition in colorscheme (key {key})'.format(
 					key=context.key),
-				problem='name {0} is not present in {1} {2} colorschemes: {3}'.format(
-					group, tofind, ext, ', '.join(not_found)),
+				problem='name {0} is not present anywhere in {1} {2} {3} colorschemes: {4}'.format(
+					group, len(not_found), ext, new_colorscheme, ', '.join(not_found)),
 				problem_mark=group.mark
 			)
-	new_echoerr.echo_all()
+			hadproblem = True
 	return True, False, hadproblem
 
 
@@ -279,7 +283,7 @@ def check_key_compatibility(segment, data, context, echoerr):
 			context_mark=context[-1][1].mark,
 			problem=(
 				'found missing keys required to determine highlight group. '
-				'Either highlight_group or name key must be present'
+				'Either highlight_groups or name key must be present'
 			)
 		)
 		hadproblem = True
@@ -342,6 +346,28 @@ def check_full_segment_data(segment, data, context, echoerr):
 	return check_key_compatibility(segment_copy, data, context, echoerr)
 
 
+highlight_group_spec = Spec().ident().copy
+_highlight_group_spec = highlight_group_spec().context_message(
+	'Error while checking function documentation while checking theme (key {key})')
+
+
+def check_hl_group_name(hl_group, context_mark, context, echoerr):
+	'''Check highlight group name: it should match naming conventions
+
+	:param str hl_group:
+		Checked group.
+	:param Mark context_mark:
+		Context mark. May be ``None``.
+	:param Context context:
+		Current context.
+	:param func echoerr:
+		Function used for error reporting.
+
+	:return: ``False`` if check succeeded and ``True`` if it failed.
+	'''
+	return _highlight_group_spec.match(hl_group, context_mark=context_mark, context=context, echoerr=echoerr)[1]
+
+
 def check_segment_function(function_name, data, context, echoerr):
 	havemarks(function_name)
 	ext = data['ext']
@@ -383,13 +409,16 @@ def check_segment_function(function_name, data, context, echoerr):
 			if r:
 				echoerr(
 					context='Error while checking theme (key {key})'.format(key=context.key),
+					context_mark=function_name.mark,
 					problem=(
 						'found highlight group {0} not defined in the following colorschemes: {1}\n'
 						'(Group name was obtained from function documentation.)'
 					).format(divider_hl_group, list_sep.join(r)),
-					problem_mark=function_name.mark
+					problem_mark=divider_hl_group.mark,
 				)
 				hadproblem = True
+			if check_hl_group_name(divider_hl_group, function_name.mark, context, echoerr):
+				hadproblem = True
 
 		if hl_groups:
 			greg = re.compile(r'``([^`]+)``( \(gradient\))?')
@@ -407,6 +436,8 @@ def check_segment_function(function_name, data, context, echoerr):
 								match.group(1),
 								Mark(*mark_args, pointer=sub_pointer + match.start(1))
 							)
+							if check_hl_group_name(hl_group, function_name.mark, context, echoerr):
+								hadproblem = True
 							gradient = bool(match.group(2))
 							required_pack.append((hl_group, gradient))
 						finally:
@@ -470,6 +501,56 @@ def check_segment_function(function_name, data, context, echoerr):
 	return True, False, False
 
 
+def hl_group_in_colorscheme(hl_group, cconfig, allow_gradients, data, context, echoerr):
+	havemarks(hl_group, cconfig)
+	if hl_group not in cconfig.get('groups', {}):
+		return False
+	elif not allow_gradients or allow_gradients == 'force':
+		group_config = cconfig['groups'][hl_group]
+		while isinstance(group_config, unicode):
+			try:
+				group_config = cconfig['groups'][group_config]
+			except KeyError:
+				# No such group. Error was already reported when checking 
+				# colorschemes.
+				return True
+		havemarks(group_config)
+		hadgradient = False
+		for ckey in ('fg', 'bg'):
+			color = group_config.get(ckey)
+			if not color:
+				# No color. Error was already reported when checking 
+				# colorschemes.
+				return True
+			havemarks(color)
+			# Gradients are only allowed for function segments. Note that 
+			# whether *either* color or gradient exists should have been 
+			# already checked
+			hascolor = color in data['colors_config'].get('colors', {})
+			hasgradient = color in data['colors_config'].get('gradients', {})
+			if hasgradient:
+				hadgradient = True
+			if allow_gradients is False and not hascolor and hasgradient:
+				echoerr(
+					context='Error while checking highlight group in theme (key {key})'.format(
+						key=context.key),
+					context_mark=hl_group.mark,
+					problem='group {0} is using gradient {1} instead of a color'.format(hl_group, color),
+					problem_mark=color.mark
+				)
+				return False
+		if allow_gradients == 'force' and not hadgradient:
+			echoerr(
+				context='Error while checking highlight group in theme (key {key})'.format(
+					key=context.key),
+				context_mark=hl_group.mark,
+				problem='group {0} should have at least one gradient color, but it has no'.format(hl_group),
+				problem_mark=group_config.mark
+			)
+			return False
+	return True
+
+
 def hl_exists(hl_group, data, context, echoerr, allow_gradients=False):
 	havemarks(hl_group)
 	ext = data['ext']
@@ -478,45 +559,14 @@ def hl_exists(hl_group, data, context, echoerr, allow_gradients=False):
 		# twice
 		return []
 	r = []
+	found = False
 	for colorscheme, cconfig in data['colorscheme_configs'][ext].items():
-		if hl_group not in cconfig.get('groups', {}):
+		if hl_group_in_colorscheme(hl_group, cconfig, allow_gradients, data, context, echoerr):
+			found = True
+		else:
 			r.append(colorscheme)
-		elif not allow_gradients or allow_gradients == 'force':
-			group_config = cconfig['groups'][hl_group]
-			havemarks(group_config)
-			hadgradient = False
-			for ckey in ('fg', 'bg'):
-				color = group_config.get(ckey)
-				if not color:
-					# No color. Error was already reported.
-					continue
-				havemarks(color)
-				# Gradients are only allowed for function segments. Note that 
-				# whether *either* color or gradient exists should have been 
-				# already checked
-				hascolor = color in data['colors_config'].get('colors', {})
-				hasgradient = color in data['colors_config'].get('gradients', {})
-				if hasgradient:
-					hadgradient = True
-				if allow_gradients is False and not hascolor and hasgradient:
-					echoerr(
-						context='Error while checking highlight group in theme (key {key})'.format(
-							key=context.key),
-						context_mark=hl_group.mark,
-						problem='group {0} is using gradient {1} instead of a color'.format(hl_group, color),
-						problem_mark=color.mark
-					)
-					r.append(colorscheme)
-					continue
-			if allow_gradients == 'force' and not hadgradient:
-				echoerr(
-					context='Error while checking highlight group in theme (key {key})'.format(
-						key=context.key),
-					context_mark=hl_group.mark,
-					problem='group {0} should have at least one gradient color, but it has no'.format(hl_group),
-					problem_mark=group_config.mark
-				)
-				r.append(colorscheme)
+	if not found:
+		pass
 	return r
 
 
@@ -673,6 +723,16 @@ def get_one_segment_function(data, context, echoerr):
 			yield func
 
 
+common_names = defaultdict(set)
+
+
+def register_common_name(name, cmodule, cname):
+	s = cmodule + '.' + cname
+	cmodule_mark = Mark('<common name definition>', 1, 1, s, 1)
+	cname_mark = Mark('<common name definition>', 1, len(cmodule) + 1, s, len(cmodule) + 1)
+	common_names[name].add((MarkedUnicode(cmodule, cmodule_mark), MarkedUnicode(cname, cname_mark)))
+
+
 def get_all_possible_functions(data, context, echoerr):
 	name = context[-2][0]
 	module, name = name.rpartition('.')[::2]
@@ -681,6 +741,11 @@ def get_all_possible_functions(data, context, echoerr):
 		if func:
 			yield func
 	else:
+		if name in common_names:
+			for cmodule, cname in common_names[name]:
+				cfunc = import_segment(cname, data, context, echoerr, module=MarkedUnicode(cmodule, None))
+				if cfunc:
+					yield cfunc
 		for ext, theme_config in list_themes(data, context):
 			for segments in theme_config.get('segments', {}).values():
 				for segment in segments:
