@@ -124,8 +124,10 @@ class Renderer(object):
 		'getcwd': getattr(os, 'getcwdu', os.getcwd),
 		'home': os.environ.get('HOME'),
 	}
-	'''Basic segment info. Is merged with local segment information by 
-	``.get_segment_info()`` method. Keys:
+	'''Basic segment info
+
+	Is merged with local segment information by :py:meth:`get_segment_info` 
+	method. Keys:
 
 	``environ``
 		Object containing environment variables. Must define at least the 
@@ -279,9 +281,10 @@ class Renderer(object):
 			string_width)``. Returns a three-tuple if ``output_raw`` is also 
 			``True``: ``(colored_string, colorless_string, string_width)``.
 		:param dict segment_info:
-			Segment information. See also ``.get_segment_info()`` method.
+			Segment information. See also :py:meth:`get_segment_info` method.
 		:param matcher_info:
-			Matcher information. Is processed in ``.get_theme()`` method.
+			Matcher information. Is processed in :py:meth:`get_segment_info` 
+			method.
 		'''
 		theme = self.get_theme(matcher_info)
 		return self.do_render(
@@ -291,7 +294,7 @@ class Renderer(object):
 			line=line,
 			output_raw=output_raw,
 			output_width=output_width,
-			segment_info=segment_info,
+			segment_info=self.get_segment_info(segment_info, mode),
 			theme=theme,
 		)
 
@@ -310,7 +313,7 @@ class Renderer(object):
 	def do_render(self, mode, width, side, line, output_raw, output_width, segment_info, theme):
 		'''Like Renderer.render(), but accept theme in place of matcher_info
 		'''
-		segments = list(theme.get_segments(side, line, self.get_segment_info(segment_info, mode), mode))
+		segments = list(theme.get_segments(side, line, segment_info, mode))
 
 		current_width = 0
 
@@ -375,7 +378,9 @@ class Renderer(object):
 		elif output_width:
 			current_width = self._render_length(theme, segments, divider_widths)
 
-		rendered_highlighted = ''.join([segment['_rendered_hl'] for segment in self._render_segments(theme, segments)]) + self.hlstyle()
+		rendered_highlighted = ''.join([segment['_rendered_hl'] for segment in self._render_segments(theme, segments)])
+		if rendered_highlighted:
+			rendered_highlighted += self.hlstyle()
 
 		return construct_returned_value(rendered_highlighted, segments, current_width, output_raw, output_width)
 
@@ -492,7 +497,7 @@ class Renderer(object):
 		'''
 		return string.translate(self.character_translations)
 
-	def hlstyle(fg=None, bg=None, attr=None):
+	def hlstyle(fg=None, bg=None, attrs=None):
 		'''Output highlight style string.
 
 		Assuming highlighted string looks like ``{style}{contents}`` this method 
@@ -501,10 +506,10 @@ class Renderer(object):
 		'''
 		raise NotImplementedError
 
-	def hl(self, contents, fg=None, bg=None, attr=None):
+	def hl(self, contents, fg=None, bg=None, attrs=None):
 		'''Output highlighted chunk.
 
-		This implementation just outputs ``.hlstyle()`` joined with 
+		This implementation just outputs :py:meth:`hlstyle` joined with 
 		``contents``.
 		'''
-		return self.hlstyle(fg, bg, attr) + (contents or '')
+		return self.hlstyle(fg, bg, attrs) + (contents or '')
